DischargeableMinDropFix = {
	prerequisitesPresent = function (specializations)
		return SpecializationUtil.hasSpecialization(Dischargeable, specializations)
	end
}

function DischargeableMinDropFix.registerOverwrittenFunctions(vehicleType)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "dischargeToGround", DischargeableMinDropFix.dischargeToGround)
	SpecializationUtil.registerOverwrittenFunction(vehicleType, "setDischargeEffectActive", DischargeableMinDropFix.setDischargeEffectActive)
end

function DischargeableMinDropFix.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", DischargeableMinDropFix)
end

function DischargeableMinDropFix:onLoad(savegame)
	self.spec_combine.threshingScale = 1.5
end

function DischargeableMinDropFix:dischargeToGround(superFunc, dischargeNode, emptyLiters)
	local fillType = self:getDischargeFillType(dischargeNode)
	local fillLevel = self:getFillUnitFillLevel(dischargeNode.fillUnitIndex)
	local minLiterToDrop = g_densityMapHeightManager:getMinValidLiterValue(fillType)
	dischargeNode.litersToDrop = math.min(dischargeNode.litersToDrop + emptyLiters, math.max(dischargeNode.emptySpeed * 250, minLiterToDrop))
	dischargeNode.litersToDrop = math.min(dischargeNode.litersToDrop, fillLevel)
	local minDropReached = minLiterToDrop < dischargeNode.litersToDrop
	local hasMinDropFillLevel = minLiterToDrop < fillLevel
	local info = dischargeNode.info
	local dischargedLiters = 0
	local sx, sy, sz = localToWorld(info.node, -info.width, 0, info.zOffset)
	local ex, ey, ez = localToWorld(info.node, info.width, 0, info.zOffset)
	sy = sy + info.yOffset
	ey = ey + info.yOffset

	if info.limitToGround then
		sy = math.max(getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, sx, 0, sz) + 0.1, sy)
		ey = math.max(getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, ex, 0, ez) + 0.1, ey)
	end

	local dropped, lineOffset = DensityMapHeightUtil.tipToGroundAroundLine(self, dischargeNode.litersToDrop, fillType, sx, sy, sz, ex, ey, ez, info.length, nil, dischargeNode.lineOffset, true, nil, true)
	dischargeNode.lineOffset = lineOffset
	dischargeNode.litersToDrop = dischargeNode.litersToDrop - dropped

	if dropped > 0 then
		local unloadInfo = self:getFillVolumeUnloadInfo(dischargeNode.unloadInfoIndex)
		dischargedLiters = self:addFillUnitFillLevel(self:getOwnerFarmId(), dischargeNode.fillUnitIndex, -dropped, fillType, ToolType.UNDEFINED, unloadInfo)
	else
		fillLevel = self:getFillUnitFillLevel(dischargeNode.fillUnitIndex)

		if fillLevel < dischargeNode.litersToDrop then
			dischargeNode.litersToDrop = fillLevel
		end
	end

	return dischargedLiters, minDropReached, hasMinDropFillLevel
end

function DischargeableMinDropFix:setDischargeEffectActive(superFunc, dischargeNode, isActive, force)
	local fillType = self:getDischargeFillType(dischargeNode)
	local fillLevel = self:getFillUnitFillLevel(dischargeNode.fillUnitIndex)
	local minLiterToDrop = g_densityMapHeightManager:getMinValidLiterValue(fillType)

	if fillLevel < minLiterToDrop then
		if g_time - (dischargeNode.lastTimeEffectValid or 0) > 1000 then
			isActive = false
		end
	else
		dischargeNode.lastTimeEffectValid = g_time
	end

	superFunc(self, dischargeNode, isActive, force)
end
